//
//  WForest.java
//  Forest
//
//  Created by Leonardo Yamasaki Maza on 16/05/08.
//  Copyright 2008 Leonardo Yamasaki Maza. All rights reserved.
//

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.applet.*;
import java.util.*;
import java.net.*;
import javax.swing.Timer;

////////////////////////
public class WForest extends JFrame implements ActionListener{
	JPanel world;
	JPanel header;
	JPanel footer;
	
	JButton Start;
	JButton Reset;
	
	String strStart=new String("Comenzar");
	String strStop=new String("Parar");
	String strReset=new String("Restablece");
	String strWindLabel = new String("Direccion del viento:");
	String strWindDir = new String("abajo a arriba");
	String strSeedTrees = new String("Siembra arboles, salvemos nuestro planeta");
	String strCopyright = new String("(c) 2008, Leonardo Yamasaki");
	
	Random rand = new Random();
	Timer runtimer;
	Color State0color = Color.green; //the tree is fine healty 
	Color State1color = Color.white ; //no tree or tree dead
	
	int SIZE=50;
	int CELLDIMX=15;
	int CELLDIMY=15;
	int MAXPOPTREE=60;
	
	int DESV=3;          //max desv
	int ALFA=1/2;        //coef of affect
	int HEIGHTMAX=9;    //Height maxim of trees
	int PBORN=10000;    //10 000 between 1 000 000 of born
	int PDEAD=100;      //100 between 1 000 000 of dead
	
	int TIMEPERIOD=500; // tiempo en milisegundos
	
	ImageIcon[] TreeIcon = new ImageIcon[10]; //Levels of tree height
	
	Cell[][] cell= new Cell[SIZE][SIZE];
	
	public static boolean WFon=false;
	
	/////////////////////////////////////////////////////////
	// Constructor
	/////////////////////////////////////////////////////////
	public WForest(String title){
		super(title);
		JPanel world = new JPanel(true);
		JPanel header = new JPanel();
		JPanel footer = new JPanel();
		
		header.setLayout(new GridLayout(1,1,0,0));
		header.add(new JLabel(title));
		
		Start = new JButton(strStart);
		Start.addActionListener(this);
		header.add(Start);
		Reset = new JButton(strReset);
		Reset.addActionListener(this);
		header.add(Reset);
		
		footer.setLayout(new GridLayout(2,2,0,0));
		footer.add(new JLabel(strWindLabel));
		footer.add(new JLabel(strWindDir));
		footer.add(new JLabel(strSeedTrees));
		footer.add(new JLabel(strCopyright));
		
		world.setLayout(new GridLayout(SIZE,SIZE,0,0));
					
		//tree icons
		String t= new String();
		for(int i=0; i<HEIGHTMAX+1; i++){
			t = "tree-icon" + String.valueOf(i) + ".gif";
			TreeIcon[i] = new ImageIcon(getClass().getResource(t));
		}
			
		
		for(int x = 0; x <  SIZE; x++){
			for(int y=0; y < SIZE; y++){
				cell[x][y] = new Cell();
				cell[x][y].setPreferredSize(new Dimension(CELLDIMX, CELLDIMY));
//				cell[x][y].setBorder(BorderFactory.createLineBorder(Color.black));
				cell[x][y].setOpaque(true);
				//set initial state
				initCellState(x,y);
				world.add(cell[x][y]);
			}
		}
		Container cp =  this.getContentPane();
		cp.setLayout(new BorderLayout());
		cp.add(header,BorderLayout.NORTH);
		cp.add(world,BorderLayout.CENTER);
		cp.add(footer,BorderLayout.SOUTH);
		this.setSize((SIZE*CELLDIMX),(SIZE*CELLDIMY));
		WForest.WFon=true;
		this.setVisible(true);
		this.setResizable(false);
	}
	
	public void actionPerformed(ActionEvent ae){
		if (ae.getSource() == Reset) {
			cellsReset();
		} else {
		if (ae.getSource()==Start && Start.getText()==strStart) {
			Start.setText(strStop);
			Reset.setEnabled(false);
			runtimer = new Timer (TIMEPERIOD, new ActionListener () 
				{ 
				public void actionPerformed(ActionEvent e) 
				{ 
					doSimulation();
				} 
			});
			runtimer.start();
		}
		else {
			Start.setText(strStart);
			Reset.setEnabled(true);
			runtimer.stop();
			}
		}
	}
	
	///////////////////////////////////////////////////////////
	void initCellState(int x, int y){
		int state=0;
		int r = rand.nextInt(100);
		int s = rand.nextInt(HEIGHTMAX+1);
		if (r>0 && r<MAXPOPTREE) {
			state=0;
		}
		else {
			state=1;
		}
		switch (state) {
			case 0:
				cell[x][y].InitState(0);
				cell[x][y].InitSTree(s);
				cell[x][y].setBackground(State0color);
//				cell[x][y].setText(String.valueOf(s));
				cell[x][y].setIcon(TreeIcon[s]);
				break;
			case 1:
				cell[x][y].InitState(1);
				cell[x][y].InitSTree(0);
				cell[x][y].setBackground(State1color);
//				cell[x][y].setText("");
				cell[x][y].setIcon(null);
				break;
		}		
	}
	
	/////////////////////////////////////////////////////////
	void cellsReset(){
		for(int x = 0; x <  SIZE; x++){
			for(int y=0; y < SIZE; y++){
				initCellState(x,y);
			}
		}
	}
	
	/////////////////////////////////////////////////////////
	void cellsNewTime(){
		for(int x = 0; x <  SIZE; x++){
			for(int y=0; y < SIZE; y++){
				cell[x][y].NewTime();
			}
		}
	}
	
	/////////////////////////////////////////////////////////
	void doSimulation(){
		int pp=0;
		int pd=0;
		int sum=0;
		for(int x=0; x <  SIZE; x++){
			for(int y=0; y < SIZE; y++){
				pd = rand.nextInt(1000000);
				pp = rand.nextInt(1000000);
				switch (cell[x][y].getState()) {
				case 0: // tree
					if (cell[x][y].getSTree()==HEIGHTMAX && pd < PDEAD) {
						changeState(x,y,1); //dead by accident
					}
					else {
						sum = (1 /(1 + (2*ALFA)))*(getNeighbor(x,y,2) + (ALFA*(getNeighbor(x,y,7) + getNeighbor(x,y,9))));
						if (sum <= DESV) {
							changeState(x,y,0);
						}
						else {
							changeState(x,y,1);
						}
					}
					break;
				case 1://land no tree
					if (pp < PBORN) {
						changeState(x,y,0);
					}
					break;
				}
			}
		}
		paintWorld();
		cellsNewTime();
	}
	
	/////////////////////////////////////////////////////////
	void stopSimulation(){		
	}
	
	int getNeighbor(int x,int y, int num){
		int temp=0;
		switch (num) {
		case 1:
			temp = getNStat(x-1,y-1);
			break;
		case 2:
			temp = getNStat(x,y-1);
			break;
		case 3:
			temp = getNStat(x+1,y-1);
			break;
		case 4:
			temp = getNStat(x-1,y);
			break;
		case 5:
			temp = getNStat(x,y);
			break;
		case 6:
			temp = getNStat(x+1,y);
			break;
		case 7:
			temp = getNStat(x-1,y+1);
			break;
		case 8:
			temp = getNStat(x+1,y+1);
			break;
		}
		return temp;
	}
	
	////////////////////////
	int getNStat(int x, int y){
		int r=0;
		if (x<0 || y<0 || x>=SIZE || y>=SIZE) {
			r = 0;
		}
		else if (cell[x][y].getState()==0) {
			r = cell[x][y].getSTree();
		}
		return r;
	}

	////////////////////////
	void changeState(int x, int y, int state){
		switch (state) {
		case 0:
			cell[x][y].setState(0);
			int actS=cell[x][y].getSTree();
			if(actS != HEIGHTMAX){
				cell[x][y].setSTree(actS+1);
				cell[x][y].setBackground(State0color);
				cell[x][y].setIcon(TreeIcon[actS+1]);
//				cell[x][y].setText(String.valueOf(actS+1));
			}
			break;
		case 1: //tree dead
			cell[x][y].setState(1);
			cell[x][y].setSTree(0);
			cell[x][y].setBackground(State1color);
			cell[x][y].setIcon(null);
//			cell[x][y].setText("");
			break;
		}
	}
	
	////////////
	void paintWorld(){
		this.getContentPane().paintComponents(getContentPane().getGraphics());
		
	}
	
}//End Class
